using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace OnlinePhotoShop
{
	/// <summary>
	/// Web form to display the shopping cart items.
	/// </summary>
	public class Cart : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.Label Total;
		protected System.Web.UI.WebControls.HyperLink HyperLink1;
		protected System.Web.UI.WebControls.HyperLink HyperLink2;
		protected System.Web.UI.WebControls.Repeater Table;
	
		/// <summary>
		/// Page initialization code.
		/// </summary>
		private void Page_Load(object sender, System.EventArgs e)
		{
			if (!Page.IsPostBack)
			{
				this.UpdateCart();
			}

			// do not cache this page
			Response.Expires = -1;
		}

		/// <summary>
		/// Generates a list of formatted shopping cart items and binds them
		/// to the repeater control.
		/// </summary>
		public void UpdateCart()
		{
			// retrieve shopping cart information from session object
			ShoppingCart cart = (ShoppingCart) Session["ShoppingCart"];
			if (cart == null)
			{
				cart = new ShoppingCart(Session.SessionID);
				Session["ShoppingCart"] = cart;
			}

			// copy shopping cart items into a formatted list
			ArrayList list = new ArrayList();
			IDictionaryEnumerator i = cart.GetEnumerator();

			double total = 0;
			while(i.MoveNext())
			{
				ShoppingItemView item = new ShoppingItemView(
					Server.MapPath("products.xml"),
					(int) i.Key,
					(ShoppingItem) i.Value);

				list.Add(item);
				total += item.Subtotal;
			}

			// bind the formatted list to the table
			Table.DataSource = list;
			Table.DataBind();

			// update total cost label
			Total.Text = total.ToString("C");
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Table.ItemCommand += new System.Web.UI.WebControls.RepeaterCommandEventHandler(this.Table_ItemCommand);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		/// <summary>
		/// Command handler for buttons pressed within the table (remove).
		/// </summary>
		/// <param name="source">Source object.</param>
		/// <param name="e">Event argument.</param>
		private void Table_ItemCommand(object source, System.Web.UI.WebControls.RepeaterCommandEventArgs e)
		{
			// remove from cart
			string cmd = (string) e.CommandArgument;
			int key = System.Convert.ToInt32(cmd);
			ShoppingCart cart = (ShoppingCart) Session["ShoppingCart"];

			if (cart != null)
			{
				System.IO.File.Delete(((ShoppingItem) cart[key]).ServerPath);
				cart.Remove(key);
				this.UpdateCart();
			}
		}
	}
}
