using System;
using System.Runtime.Serialization;
using System.Security.Permissions;

namespace OnlinePhotoShop
{
	/// <summary>
	/// Class that contains all information to describe one item in the shopping cart (product id, option id, quantity and image locations).
	/// </summary>
	/// <requirement>F:product_shopping_cart</requirement>
	/// <requirement>F:product_customize</requirement>
	/// <requirement>F:product_options</requirement>
	[Serializable()]
	public class ShoppingItem : ISerializable
	{
		#region Do It Yourself Chapter 11
		/// <summary>
		/// Private memeber for product identifier.
		/// </summary>
		string mProduct;
		/// <summary>
		/// Private memeber for option identifier.
		/// </summary>
		string mOption;
		/// <summary>
		/// Private memeber for quantity.
		/// </summary>
		int mQuantity;
		/// <summary>
		/// Private memeber to store the client path of the associated image.
		/// </summary>
		string mClientPath;
		/// <summary>
		/// Private memeber to store the server path of an uploaded image.
		/// </summary>
		string mServerPath;

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="product">Product identifier.</param>
		/// <param name="option">Option identifier.</param>
		/// <param name="quantity">Ordered quantity of this item.</param>
		/// <param name="clientPath">Path of the associated image on the client.</param>
		/// <param name="serverPath">Path of the uploaded image.</param>
		public ShoppingItem(string product, string option, int quantity, string clientPath, string serverPath)
		{
			mProduct = product;
			mOption = option;
			mQuantity = quantity;
			mClientPath = clientPath;
			mServerPath = serverPath;
		}

		/// <summary>
		/// Product identifier.
		/// </summary>
		public string Product
		{
			get 
			{
				return mProduct;
			}
		}

		/// <summary>
		/// Option identifier.
		/// </summary>
		public string Option
		{
			get 
			{
				return mOption;
			}
		}

		/// <summary>
		/// Quantity.
		/// </summary>
		public int Quantity
		{
			get 
			{
				return mQuantity;
			}
		}

		/// <summary>
		/// Image path on the client.
		/// </summary>
		public string ClientPath
		{
			get 
			{
				return mClientPath;
			}
		}

		/// <summary>
		/// Image path on the server.
		/// </summary>
		public string ServerPath
		{
			get 
			{
				return mServerPath;
			}
		}
		#endregion

		#region ISerializable Members

		/// <summary>
		/// The serialization function.
		/// </summary>
		/// <param name="info">The object to be populated with serialization information.</param>
		/// <param name="context">The destination context of the serialization.</param>
		[SecurityPermissionAttribute(SecurityAction.Demand,SerializationFormatter=true)]
		public void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("Product", mProduct);
			info.AddValue("Option", mOption);
			info.AddValue("Quantity", mQuantity);
			info.AddValue("ClientPath", mClientPath);
			info.AddValue("ServerPath", mServerPath);
		}

		/// <summary>
		/// The deserialization constructor.
		/// </summary>
		/// <param name="info">The object populated with serialization information.</param>
		/// <param name="context">The source context of the serialization.</param>
		public ShoppingItem(SerializationInfo info, StreamingContext context) 
		{
			mProduct = (string) info.GetValue("Product", typeof(string));
			mOption = (string) info.GetValue("Option", typeof(string));
			mQuantity = (int) info.GetValue("Quantity", typeof(int));
			mClientPath = (string) info.GetValue("ClientPath", typeof(string));
			mServerPath = (string) info.GetValue("ServerPath", typeof(string));
		}

		#endregion

	}
}
